@@ -132,6 +132,25 @@ alloc_string(GladeInterface *interface, const gchar *string)
     return s;
 }
 
+static gchar *
+alloc_propname(GladeInterface *interface, const gchar *string)
+{
+    static GString *norm_str;
+    gint i;
+
+    if (!norm_str)
+	norm_str = g_string_new_len(NULL, 64);
+
+    /* assign the string to norm_str */
+    g_string_assign(norm_str, string);
+    /* convert all dashes to underscores */
+    for (i = 0; i < norm_str->len; i++)
+	if (norm_str->str[i] == '-')
+	    norm_str->str[i] = '_';
+
+    return alloc_string(interface, norm_str->str);
+}
+
 static GladeWidgetInfo *
 create_widget_info(GladeInterface *interface, const xmlChar **attrs)
 {
@@ -140,13 +159,13 @@ create_widget_info(GladeInterface *interface, const xmlChar **attrs)
 
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 	if (!strcmp(attrs[i], "class"))
-	    info->class = alloc_string(interface, attrs[i+1]);
+	    info->classname = alloc_string(interface, attrs[i+1]);
 	else if (!strcmp(attrs[i], "id"))
 	    info->name = alloc_string(interface, attrs[i+1]);
 	else
 	    g_warning("unknown attribute `%s' for <widget>.", attrs[i]);
     }
-    if (info->class == NULL || info->name == NULL)
+    if (info->classname == NULL || info->name == NULL)
 	g_warning("<widget> element missing required attributes!");
     g_hash_table_insert(interface->names, info->name, info);
     return info;
@@ -555,23 +574,33 @@ glade_parser_start_element(GladeParseState *state,
 	break;
     case PARSER_WIDGET:
 	if (!strcmp(name, "property")) {
+	    gboolean bad_agent = FALSE;
+
 	    if (state->prop_type != PROP_NONE &&
 		state->prop_type != PROP_WIDGET)
 		g_warning("non widget properties defined here (oh no!)");
-	    state->prop_type = PROP_WIDGET;
 	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!strcmp(attrs[i], "name"))
-		    state->prop_name = alloc_string(state->interface,
-						    attrs[i+1]);
-		else if (!strcmp(attrs[i], "translatable") &&
-			 !strcmp(attrs[i+1], "yes"))
-		    state->translate_prop = TRUE;
+		    state->prop_name = alloc_propname(state->interface,
+						      attrs[i+1]);
+		else if (!strcmp(attrs[i], "translatable"))
+		    state->translate_prop = !strcmp(attrs[i+1], "yes");
+		else if (!strcmp(attrs[i], "agent"))
+		    bad_agent = strcmp(attrs[i], "libglade") != 0;
 		else
 		    g_warning("unknown attribute `%s' for <property>.",
 			      attrs[i]);
 	    }
-	    state->state = PARSER_WIDGET_PROPERTY;
+	    if (bad_agent) {
+		/* ignore the property ... */
+		state->prev_state = state->state;
+		state->state = PARSER_UNKNOWN;
+		state->unknown_depth++;
+	    } else {
+		state->prop_type = PROP_WIDGET;
+		state->state = PARSER_WIDGET_PROPERTY;
+	    }
 	} else if (!strcmp(name, "accessibility")) {
 	    flush_properties(state);
 
@@ -609,11 +638,10 @@ glade_parser_start_element(GladeParseState *state,
 	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!strcmp(attrs[i], "name"))
-		    state->prop_name = alloc_string(state->interface,
-						    attrs[i+1]);
-		else if (!strcmp(attrs[i], "translatable") &&
-			 !strcmp(attrs[i+1], "yes"))
-		    state->translate_prop = TRUE;
+		    state->prop_name = alloc_propname(state->interface,
+						      attrs[i+1]);
+		else if (!strcmp(attrs[i], "translatable"))
+		    state->translate_prop = !strcmp(attrs[i+1], "yes");
 		else
 		    g_warning("unknown attribute `%s' for <atkproperty>.",
 			      attrs[i]);
@@ -751,23 +779,33 @@ glade_parser_start_element(GladeParseState *state,
 	break;
     case PARSER_WIDGET_CHILD_PACKING:
 	if (!strcmp(name, "property")) {
+	    gboolean bad_agent = FALSE;
+
 	    if (state->prop_type != PROP_NONE &&
 		state->prop_type != PROP_CHILD)
 		g_warning("non child properties defined here (oh no!)");
-	    state->prop_type = PROP_CHILD;
 	    state->translate_prop = FALSE;
 	    for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 		if (!strcmp(attrs[i], "name"))
-		    state->prop_name = alloc_string(state->interface,
-						    attrs[i+1]);
-		else if (!strcmp(attrs[i], "translatable") &&
-			 !strcmp(attrs[i+1], "yes"))
-		    state->translate_prop = TRUE;
+		    state->prop_name = alloc_propname(state->interface,
+						      attrs[i+1]);
+		else if (!strcmp(attrs[i], "translatable"))
+		    state->translate_prop = !strcmp(attrs[i+1], "yes");
+		else if (!strcmp(attrs[i], "agent"))
+		    bad_agent = strcmp(attrs[i], "libglade") != 0;
 		else
 		    g_warning("unknown attribute `%s' for <property>.",
 			      attrs[i]);
 	    }
-	    state->state = PARSER_WIDGET_CHILD_PACKING_PROPERTY;
+	    if (bad_agent) {
+		/* ignore the property ... */
+		state->prev_state = state->state;
+		state->state = PARSER_UNKNOWN;
+		state->unknown_depth++;
+	    } else {
+		state->prop_type = PROP_CHILD;
+		state->state = PARSER_WIDGET_CHILD_PACKING_PROPERTY;
+	    }
 	} else {
 	    g_warning("Unexpected element <%s> inside <child>.", name);
 	    state->prev_state = state->state;
@@ -1195,7 +1233,7 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
     xmlNode *widget = xmlNewNode(NULL, "widget");
     gint i, j;
 
-    xmlSetProp(widget, "class", info->class);
+    xmlSetProp(widget, "class", info->classname);
     xmlSetProp(widget, "id", info->name);
     xmlAddChild(parent, widget);
     xmlNodeAddContent(widget, "\n");
